#define TRACE_POSTGRESQL_TRANSACTION_START(INT1)
#define TRACE_POSTGRESQL_TRANSACTION_START_ENABLED() (0)
#define TRACE_POSTGRESQL_TRANSACTION_COMMIT(INT1)
#define TRACE_POSTGRESQL_TRANSACTION_COMMIT_ENABLED() (0)
#define TRACE_POSTGRESQL_TRANSACTION_ABORT(INT1)
#define TRACE_POSTGRESQL_TRANSACTION_ABORT_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_RELEASE(INT1)
#define TRACE_POSTGRESQL_LWLOCK_RELEASE_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_WAIT_START(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_WAIT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_WAIT_DONE(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_WAIT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_CONDACQUIRE(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_CONDACQUIRE_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_CONDACQUIRE_FAIL(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_CONDACQUIRE_FAIL_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE_OR_WAIT(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE_OR_WAIT_ENABLED() (0)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE_OR_WAIT_FAIL(INT1, INT2)
#define TRACE_POSTGRESQL_LWLOCK_ACQUIRE_OR_WAIT_FAIL_ENABLED() (0)
#define TRACE_POSTGRESQL_LOCK_WAIT_START(INT1, INT2, INT3, INT4, INT5, INT6)
#define TRACE_POSTGRESQL_LOCK_WAIT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_LOCK_WAIT_DONE(INT1, INT2, INT3, INT4, INT5, INT6)
#define TRACE_POSTGRESQL_LOCK_WAIT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_PARSE_START(INT1)
#define TRACE_POSTGRESQL_QUERY_PARSE_START_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_PARSE_DONE(INT1)
#define TRACE_POSTGRESQL_QUERY_PARSE_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_REWRITE_START(INT1)
#define TRACE_POSTGRESQL_QUERY_REWRITE_START_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_REWRITE_DONE(INT1)
#define TRACE_POSTGRESQL_QUERY_REWRITE_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_PLAN_START()
#define TRACE_POSTGRESQL_QUERY_PLAN_START_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_PLAN_DONE()
#define TRACE_POSTGRESQL_QUERY_PLAN_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_EXECUTE_START()
#define TRACE_POSTGRESQL_QUERY_EXECUTE_START_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_EXECUTE_DONE()
#define TRACE_POSTGRESQL_QUERY_EXECUTE_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_START(INT1)
#define TRACE_POSTGRESQL_QUERY_START_ENABLED() (0)
#define TRACE_POSTGRESQL_QUERY_DONE(INT1)
#define TRACE_POSTGRESQL_QUERY_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_STATEMENT_STATUS(INT1)
#define TRACE_POSTGRESQL_STATEMENT_STATUS_ENABLED() (0)
#define TRACE_POSTGRESQL_SORT_START(INT1, INT2, INT3, INT4, INT5, INT6)
#define TRACE_POSTGRESQL_SORT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_SORT_DONE(INT1, INT2)
#define TRACE_POSTGRESQL_SORT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_READ_START(INT1, INT2, INT3, INT4, INT5, INT6, INT7)
#define TRACE_POSTGRESQL_BUFFER_READ_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_READ_DONE(INT1, INT2, INT3, INT4, INT5, INT6, INT7, INT8)
#define TRACE_POSTGRESQL_BUFFER_READ_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_FLUSH_START(INT1, INT2, INT3, INT4, INT5)
#define TRACE_POSTGRESQL_BUFFER_FLUSH_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_FLUSH_DONE(INT1, INT2, INT3, INT4, INT5)
#define TRACE_POSTGRESQL_BUFFER_FLUSH_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_START(INT1)
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_SYNC_START()
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_SYNC_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_DONE()
#define TRACE_POSTGRESQL_BUFFER_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_SYNC_START(INT1, INT2)
#define TRACE_POSTGRESQL_BUFFER_SYNC_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_SYNC_WRITTEN(INT1)
#define TRACE_POSTGRESQL_BUFFER_SYNC_WRITTEN_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_SYNC_DONE(INT1, INT2, INT3)
#define TRACE_POSTGRESQL_BUFFER_SYNC_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_WRITE_DIRTY_START(INT1, INT2, INT3, INT4, INT5)
#define TRACE_POSTGRESQL_BUFFER_WRITE_DIRTY_START_ENABLED() (0)
#define TRACE_POSTGRESQL_BUFFER_WRITE_DIRTY_DONE(INT1, INT2, INT3, INT4, INT5)
#define TRACE_POSTGRESQL_BUFFER_WRITE_DIRTY_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_DEADLOCK_FOUND()
#define TRACE_POSTGRESQL_DEADLOCK_FOUND_ENABLED() (0)
#define TRACE_POSTGRESQL_CHECKPOINT_START(INT1)
#define TRACE_POSTGRESQL_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_CHECKPOINT_DONE(INT1, INT2, INT3, INT4, INT5)
#define TRACE_POSTGRESQL_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_CLOG_CHECKPOINT_START(INT1)
#define TRACE_POSTGRESQL_CLOG_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_CLOG_CHECKPOINT_DONE(INT1)
#define TRACE_POSTGRESQL_CLOG_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_SUBTRANS_CHECKPOINT_START(INT1)
#define TRACE_POSTGRESQL_SUBTRANS_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_SUBTRANS_CHECKPOINT_DONE(INT1)
#define TRACE_POSTGRESQL_SUBTRANS_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_MULTIXACT_CHECKPOINT_START(INT1)
#define TRACE_POSTGRESQL_MULTIXACT_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_MULTIXACT_CHECKPOINT_DONE(INT1)
#define TRACE_POSTGRESQL_MULTIXACT_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_TWOPHASE_CHECKPOINT_START()
#define TRACE_POSTGRESQL_TWOPHASE_CHECKPOINT_START_ENABLED() (0)
#define TRACE_POSTGRESQL_TWOPHASE_CHECKPOINT_DONE()
#define TRACE_POSTGRESQL_TWOPHASE_CHECKPOINT_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_SMGR_MD_READ_START(INT1, INT2, INT3, INT4, INT5, INT6)
#define TRACE_POSTGRESQL_SMGR_MD_READ_START_ENABLED() (0)
#define TRACE_POSTGRESQL_SMGR_MD_READ_DONE(INT1, INT2, INT3, INT4, INT5, INT6, INT7, INT8)
#define TRACE_POSTGRESQL_SMGR_MD_READ_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_SMGR_MD_WRITE_START(INT1, INT2, INT3, INT4, INT5, INT6)
#define TRACE_POSTGRESQL_SMGR_MD_WRITE_START_ENABLED() (0)
#define TRACE_POSTGRESQL_SMGR_MD_WRITE_DONE(INT1, INT2, INT3, INT4, INT5, INT6, INT7, INT8)
#define TRACE_POSTGRESQL_SMGR_MD_WRITE_DONE_ENABLED() (0)
#define TRACE_POSTGRESQL_WAL_INSERT(INT1, INT2)
#define TRACE_POSTGRESQL_WAL_INSERT_ENABLED() (0)
#define TRACE_POSTGRESQL_WAL_SWITCH()
#define TRACE_POSTGRESQL_WAL_SWITCH_ENABLED() (0)
#define TRACE_POSTGRESQL_WAL_BUFFER_WRITE_DIRTY_START()
#define TRACE_POSTGRESQL_WAL_BUFFER_WRITE_DIRTY_START_ENABLED() (0)
#define TRACE_POSTGRESQL_WAL_BUFFER_WRITE_DIRTY_DONE()
#define TRACE_POSTGRESQL_WAL_BUFFER_WRITE_DIRTY_DONE_ENABLED() (0)
